// This is a demo for the CHESS.H C/C++ header.
// It will not be used during the evaluation process.
// The Black King moves randomly, but the moves are valid!
// During the evaluation process another algorithm will be used for the Black King.
// Please do not use in your program the variables defined in this file!
//   The variables in the header used during the evaluation process have
//   different identifiers, so your source might not be compiled correctly!
// The time needed for the execution of the functions in the header used for
//   evaluation will be slightly longer than the time needed for the
//   execution of the functions in this header!
// This header contains implementations of algorithms for determining legal
//   moves, performing moves, detecting mate positions etc. You may copy
//   these algorithms from this header, but you are adviced to build your own
//   algorithms (for efficiency reasons)!
// The initial positions and the number of moves are read from the CHESS.IN
//   file. You may edit your own CHESS.IN file in order to test your program!
//   Your program must not try the open the CHESS.IN file for reading because
//   this file will not exist during the evaluation. The header used for
//   evaluation will use a different method for getting the initial position
//   and the number of moves!
// This file is freeware. It cannot be modified, but can be distributed freely.
// This file is distributed as-done.
// The GInfo Editorial Stuff is not responsible
//   for any damage caused by the use of this file!
// Author: Mihai Scortaru
// Please report any bugs found in this file not later than
//   Tuesday, November 5th, 2002!

#include <stdio.h>
#include <time.h>
#include <stdlib.h>

char columnBlackKing, columnWhiteKing, columnWhiteQueen;
int lineBlackKing, lineWhiteKing, lineWhiteQueen;
int movesLeft;
int table[8][8];
int white;
int nr;
int pozx[8], pozy[8];
int initialized=0;
char ss[3];
FILE *f;

void error(char*);
void Init();
char* GetBlackKingPosition();
char* GetWhiteKingPosition();
char* GetWhiteQueenPosition();
int MovesLeft();
void MoveWhiteQueen(char*);
void MoveWhiteKing(char*);
void MoveBlackKing();
void MakeBlackKingMove(void);

void error(char *s){
  puts(s);
  exit(0);
}

void Init(){
  if (initialized)
    error("Game already initialized!");
  randomize();
  puts("Game initialized!");
  f=fopen("CHESS.IN","rt");
  fscanf(f,"%c%d %c%d %c%d %d",&columnBlackKing, &lineBlackKing,
			       &columnWhiteKing, &lineWhiteKing,
			       &columnWhiteQueen, &lineWhiteQueen,
			       &movesLeft);
  fclose(f);
  initialized = 1;
  white = 1;
}

char* GetBlackKingPosition(){
  if (!initialized)
    error("Game not initialized!");
  sprintf(ss,"%c%d",columnBlackKing,lineBlackKing);
  printf("Black King at %s!\n",ss);
  return ss;
}

char* GetWhiteKingPosition(){
  if (!initialized)
    error("Game not initialized!");
  sprintf(ss,"%c%d",columnWhiteKing,lineWhiteKing);
  printf("White King at %s!\n",ss);
  return ss;
}

char* GetWhiteQueenPosition(){
  if (!initialized)
    error("Game not initialized!");
  sprintf(ss,"%c%d",columnWhiteQueen,lineWhiteQueen);
  printf("White Queen at %s!\n",ss);
  return ss;
}

int MovesLeft(){
  if (!initialized)
    error("Game not initialized!");
  printf("%d moves left!\n",movesLeft);
  return movesLeft;
}

void MakeBlackKingMove(){
  int x,y,xx,yy,i,j,k;
  int dx[8]={1,1,1,0,-1,-1,-1,0};
  int dy[8]={1,0,-1,-1,-1,0,1,1};
  for(i=0;i<8;i++)
    for(j=0;j<8;j++)
     table[i][j]=0;
  xx = columnBlackKing - 'a';
  yy = lineBlackKing - 1;
  for (k=0;k<8;k++){
    x = xx+dx[k];
    y = yy+dy[k];
    if (x>=0 && x<8 && y>=0 && y<8)
      table[y][x] = 1;
  }
  xx = columnWhiteKing - 'a';
  yy = lineWhiteKing - 1;
  for (k=0;k<8;k++){
    x = xx+dx[k];
    y = yy+dy[k];
    if (x>=0 && x<8 && y>=0 && y<8)
      table[y][x] = 0;
  }
  xx = columnWhiteQueen - 'a';
  yy = lineWhiteQueen - 1;
  x = xx - 1;
  y = yy;
  while (x>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y][x--] = 2;
  }
  x = xx + 1;
  y = yy;
  while (x<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y][x++] = 2;
  }
  x = xx;
  y = yy - 1;
  while (y>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y--][x] = 2;
  }
  x = xx;
  y = yy + 1;
  while (y<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y++][x] = 2;
  }
  x = xx - 1;
  y = yy - 1;
  while (x>=0 && y>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y--][x--] = 2;
  }
  x = xx + 1;
  y = yy + 1;
  while (x<8 && y<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y++][x++] = 2;
  }
  x = xx - 1;
  y = yy + 1;
  while (x>=0 && y<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y++][x--] = 2;
  }
  x = xx + 1;
  y = yy - 1;
  while (x<8 && y>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    table[y--][x++] = 2;
  }
  nr = 0;
  for (i=0;i<8;i++)
    for (j=0;j<8;j++)
      if (table[i][j]==1){
	pozx[nr] = j;
	pozy[nr++] = i;
      }
}

void MoveWhiteQueen(char* position){
  int x,y,xx,yy,i,j;
  if (!initialized)
    error("Game not initialized!");
  if (!white)
    error("White player moved twice!");
  if (position[0]<'a' || position[0]>'h' || position[1]<'1' || position[1]>'8')
    error("Illegal position!");
  for(i=0;i<8;i++)
    for(j=0;j<8;j++)
     table[i][j]=0;
  xx = columnWhiteQueen - 'a';
  yy = lineWhiteQueen - 1;
  x = xx - 1;
  y = yy;
  while (x>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y][x--] = 1;
  }
  x = xx + 1;
  y = yy;
  while (x<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y][x++] = 1;
  }
  x = xx;
  y = yy - 1;
  while (y>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y--][x] = 1;
  }
  x = xx;
  y = yy + 1;
  while (y<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y++][x] = 1;
  }
  x = xx - 1;
  y = yy - 1;
  while (x>=0 && y>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y--][x--] = 1;
  }
  x = xx + 1;
  y = yy + 1;
  while (x<8 && y<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y++][x++] = 1;
  }
  x = xx - 1;
  y = yy + 1;
  while (x>=0 && y<8){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y++][x--] = 1;
  }
  x = xx + 1;
  y = yy - 1;
  while (x<8 && y>=0){
    if ((x==columnWhiteKing - 'a') && (y==lineWhiteKing - 1))
      break;
    if ((x==columnBlackKing - 'a') && (y==lineBlackKing - 1)){
      table[y][x] = 1;
      break;
    }
    table[y--][x++] = 1;
  }
  x = position[0]-'a';
  y = position[1]-'1';
  if (!table[y][x])
    error("Illegal move!");
  columnWhiteQueen = position[0];
  lineWhiteQueen = position[1] - '0';
  printf("White Queen moved to %s!\n",position);
  white = 0;
  movesLeft--;
  MakeBlackKingMove();
  if (!nr){
    x = columnBlackKing - 'a';
    y = lineBlackKing - 1;
    if (table[y][x] == 2)
      error("Check-mate! You win!");
    else
      error("Draw! Black King cannot move!");
  }
  if (!movesLeft)
    error("No more moves available!");
}

void MoveWhiteKing(char *position){
  int x,y,xx,yy,k,i,j;
  int dx[8]={1,1,1,0,-1,-1,-1,0};
  int dy[8]={1,0,-1,-1,-1,0,1,1};
  if (!initialized)
    error("Game not initialized!");
  if (!white)
    error("White player moved twice!");
  if (position[0]<'a' || position[0]>'h' || position[1]<'1' || position[1]>'8')
    error("Illegal position!");
  for(i=0;i<8;i++)
    for(j=0;j<8;j++)
      table[i][j]=0;
  xx = columnWhiteKing - 'a';
  yy = lineWhiteKing - 1;
  for (k=0;k<8;k++){
    x = xx+dx[k];
    y = yy+dy[k];
    if (x>=0 && x<8 && y>=0 && y<8)
      table[y][x] = 1;
  }
  xx = columnWhiteQueen - 'a';
  yy = lineWhiteQueen - 1;
  table[yy][xx] = 0;
  xx = columnBlackKing - 'a';
  yy = lineBlackKing - 1;
  for (k=0;k<8;k++){
    x = xx+dx[k];
    y = yy+dy[k];
    if (x>=0 && x<8 && y>=0 && y<8)
      table[y][x] = 0;
  }
  x = position[0]-'a';
  y = position[1]-'1';
  if (!table[y][x])
    error("Illegal move!");
  columnWhiteKing = position[0];
  lineWhiteKing = position[1] - '0';
  printf("White King moved to %s!\n",position);
  white = 0;
  movesLeft--;
  MakeBlackKingMove();
  if (!nr)
    error("Draw! Black King cannot move!");
  if (!movesLeft)
    error("No more moves available!");
}

void MoveBlackKing(){
  int mutare;
  if (!initialized)
    error("Game not initialized!");
  if (white)
    error("Black player asked to move twice!");
  mutare = random(nr);
  columnBlackKing = pozx[mutare]+'a';
  lineBlackKing = pozy[mutare] + 1;
  white = 1;
  puts("Black King moved!");
  if (columnBlackKing == columnWhiteQueen && lineBlackKing == lineWhiteQueen)
    error("Draw! White Queen captured!");
}
