#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>

int *edges, *costs;
char *results;
long *nodes;
int numberOfNodes;
int callsLeft;
int numberOfResults;
int initialized;
FILE *f;

void error(char *s){
  puts(s);
  exit(0);
}

void Init(){
  int i,x,y,c;
  if (initialized)
    error("Library already initilized!");
  initialized = 1;
  numberOfResults = 0;
  f = fopen("GRAPH.TXT","rt");
  fscanf(f,"%d%d",&numberOfNodes,&callsLeft);
  edges = (int*)malloc((numberOfNodes+1)*sizeof(int));
  costs = (int*)malloc((numberOfNodes+1)*sizeof(int));
  nodes = (long*)malloc((numberOfNodes+1)*sizeof(long));
  results = (char*)malloc((numberOfNodes+1)*sizeof(char));
  for (i=1;i<=numberOfNodes;i++){
    nodes[i]=1;
    edges[i] = costs[i] = results[i] = 0;
  }
  for (i=0;i<numberOfNodes;i++){
    fscanf(f,"%d%d%d",&x,&y,&c);
    nodes[x]*=c;
    nodes[y]*=c;
    if (!edges[x]){
      edges[x]=y;
      costs[x]=c;
    }
    else{
      edges[y]=x;
      costs[y]=c;
    }
  }
  fclose(f);
}

int GetNumberOfNodes(){
  if (!initialized)
    error("Library not initialized!");
  return numberOfNodes;
}

int GetNumberOfCallsLeft(){
  if (!initialized)
    error("Library not initialized!");
  return callsLeft;
}

int IsEdgeBetween(int x, int y){
  if (!initialized)
    error("Library not initialized!");
  if (x<1 || x>numberOfNodes || y<1 || y>numberOfNodes)
    error("The node does not exist!");
  return edges[x]==y || edges[y]==x;
}

int CheckEdgeWeight(int x, int y, int c){
  if (!initialized)
    error("Library not initialized!");
  if (!callsLeft)
    error("Too many calls!");
  if (x<1 || x>numberOfNodes || y<1 || y>numberOfNodes)
    error("The node does not exist!");
  callsLeft--;
  if (edges[x]==y)
    return costs[x]==c;
  else
    if (edges[y]==x)
      return costs[y]==c;
    else
      error("The edge does not exist!");
}

long GetNodeWeight(int x){
  if (!initialized)
    error("Library not initialized!");
  if (x<1 || x>numberOfNodes)
    error("The node does not exist!");
  return nodes[x];
}

void Result(int x, int y, int c){
  if (!initialized)
    error("Library not initialized!");
  if (x<1 || x>numberOfNodes || y<1 || y>numberOfNodes)
    error("The node does not exist!");
  if (edges[x]==y)
    if (costs[x]==c)
      if (!results[x]){
	results[x]=1;
	numberOfResults++;
      }
      else
	error("Result submitted twice!");
    else
      error("Wrong result!");
  else
    if (edges[y]==x)
      if (costs[y]==c)
	if (!results[y]){
	  results[y]=1;
	  numberOfResults++;
	}
	else
	  error("Result submitted twice!");
      else
	error("Wrong result!");
    else
      error("The edge does not exist!");
  if (numberOfResults == numberOfNodes)
    error("Ok!");
}
